@@ -29,12 +29,12 @@
 #include <time.h>
 
 #include "../../core/weechat.h"
-#include "../../core/wee-config.h"
-#include "../../core/wee-hashtable.h"
-#include "../../core/wee-hook.h"
-#include "../../core/wee-list.h"
-#include "../../core/wee-string.h"
-#include "../../core/wee-utf8.h"
+#include "../../core/core-config.h"
+#include "../../core/core-hashtable.h"
+#include "../../core/core-hook.h"
+#include "../../core/core-list.h"
+#include "../../core/core-string.h"
+#include "../../core/core-utf8.h"
 #include "../../plugins/plugin.h"
 #include "../gui-buffer.h"
 #include "../gui-color.h"
@@ -669,7 +669,7 @@ gui_color_init_vars ()
         gui_color_num_pairs = (gui_color_term_color_pairs >= 32768) ?
             32767 : gui_color_term_color_pairs - 1;
         gui_color_pairs = calloc (
-            (gui_color_term_colors + 2) * (gui_color_term_colors + 2),
+            (size_t)(gui_color_term_colors + 2) * (gui_color_term_colors + 2),
             sizeof (gui_color_pairs[0]));
         gui_color_pairs_used = 0;
 
@@ -763,10 +763,8 @@ gui_color_init_pairs_weechat ()
                     init_pair (i, i, -1);
             }
         }
-        if (foregrounds)
-            free (foregrounds);
-        if (backgrounds)
-            free (backgrounds);
+        free (foregrounds);
+        free (backgrounds);
     }
 }
 
@@ -1226,7 +1224,7 @@ gui_color_reset_pairs ()
     if (gui_color_pairs)
     {
         memset (gui_color_pairs, 0,
-                (gui_color_term_colors + 2)
+                (size_t)(gui_color_term_colors + 2)
                 * (gui_color_term_colors + 2)
                 * sizeof (gui_color_pairs[0]));
         gui_color_pairs_used = 0;
@@ -1297,8 +1295,7 @@ gui_color_buffer_assign ()
 {
     if (!gui_color_buffer)
     {
-        gui_color_buffer = gui_buffer_search_by_name (NULL,
-                                                      GUI_COLOR_BUFFER_NAME);
+        gui_color_buffer = gui_buffer_search (NULL, GUI_COLOR_BUFFER_NAME);
         if (gui_color_buffer)
         {
             gui_color_buffer->input_callback = &gui_color_buffer_input_cb;
@@ -1338,8 +1335,7 @@ gui_color_buffer_open ()
         if (gui_color_buffer && !gui_color_buffer->short_name)
             gui_color_buffer->short_name = strdup (GUI_COLOR_BUFFER_NAME);
 
-        if (properties)
-            hashtable_free (properties);
+        hashtable_free (properties);
     }
 
     if (!gui_color_buffer)
@@ -1534,8 +1530,7 @@ gui_color_palette_free (struct t_gui_color_palette *color_palette)
     if (!color_palette)
         return;
 
-    if (color_palette->alias)
-        free (color_palette->alias);
+    free (color_palette->alias);
 
     free (color_palette);
 }
@@ -1640,7 +1635,7 @@ void
 gui_color_dump ()
 {
     char str_line[1024];
-    int fg, bg, index, used;
+    int fg, bg, index;
 
     gui_color_info_term_colors (str_line, sizeof (str_line));
 
@@ -1652,7 +1647,6 @@ gui_color_dump ()
                      gui_color_num_pairs - gui_color_pairs_used);
     if (gui_color_pairs)
     {
-        used = 0;
         for (bg = -1; bg <= gui_color_term_colors; bg++)
         {
             for (fg = -1; fg <= gui_color_term_colors; fg++)
@@ -1663,7 +1657,6 @@ gui_color_dump ()
                     gui_chat_printf (NULL,
                                      "  fg:%3d, bg:%3d, pairs[%05d] = %hd",
                                      fg, bg, index, gui_color_pairs[index]);
-                    used++;
                 }
             }
         }
