@@ -1,7 +1,7 @@
 /*
  * gui-curses-key.c - keyboard functions for Curses GUI
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -29,11 +29,11 @@
 #include <time.h>
 
 #include "../../core/weechat.h"
-#include "../../core/wee-config.h"
-#include "../../core/wee-hook.h"
-#include "../../core/wee-log.h"
-#include "../../core/wee-utf8.h"
-#include "../../core/wee-string.h"
+#include "../../core/core-config.h"
+#include "../../core/core-hook.h"
+#include "../../core/core-log.h"
+#include "../../core/core-utf8.h"
+#include "../../core/core-string.h"
 #include "../../plugins/plugin.h"
 #include "../gui-key.h"
 #include "../gui-buffer.h"
@@ -46,7 +46,7 @@
 #include "gui-curses.h"
 
 #define BIND(key, command)                                      \
-    gui_key_default_bind(context, key, command, create_option)
+    gui_key_default_bind (context, key, command, create_option)
 
 
 /*
@@ -83,7 +83,8 @@ gui_key_default_bindings (int context, int create_option)
         BIND("meta-return",       "/input insert \\n");
         BIND("tab",               "/input complete_next");
         BIND("shift-tab",         "/input complete_previous");
-        BIND("ctrl-r",            "/input search_text_here");
+        BIND("ctrl-r",            "/input search_history");
+        BIND("ctrl-s",            "/input search_text_here");
         BIND("backspace",         "/input delete_previous_char");
         BIND("ctrl-_",            "/input undo");
         BIND("meta-_",            "/input redo");
@@ -123,6 +124,7 @@ gui_key_default_bindings (int context, int create_option)
         BIND("down",              "/input history_next");
         BIND("ctrl-up",           "/input history_global_previous");
         BIND("ctrl-down",         "/input history_global_next");
+        BIND("ctrl-o",            "/input history_use_get_next");
         BIND("shift-up",          "/input move_previous_line");
         BIND("shift-down",        "/input move_next_line");
         BIND("meta-a",            "/buffer jump smart");
@@ -137,9 +139,10 @@ gui_key_default_bindings (int context, int create_option)
         BIND("meta-k",            "/input grab_key_command");
         BIND("meta-s",            "/mute spell toggle");
         BIND("meta-u",            "/window scroll_unread");
-        BIND("ctrl-s,ctrl-u",     "/allbuf /buffer set unread");
+        BIND("meta-U",            "/allbuf /buffer set unread");
         BIND("ctrl-c,b",          "/input insert \\x02");
         BIND("ctrl-c,c",          "/input insert \\x03");
+        BIND("ctrl-c,d",          "/input insert \\x04");
         BIND("ctrl-c,i",          "/input insert \\x1D");
         BIND("ctrl-c,o",          "/input insert \\x0F");
         BIND("ctrl-c,v",          "/input insert \\x16");
@@ -205,15 +208,22 @@ gui_key_default_bindings (int context, int create_option)
             BIND(key_str, command);
         }
     }
-    else if (context == GUI_KEY_CONTEXT_SEARCH)
+    else if ((context == GUI_KEY_CONTEXT_SEARCH)
+             || (context == GUI_KEY_CONTEXT_HISTSEARCH))
     {
         BIND("return", "/input search_stop_here");
         BIND("ctrl-q", "/input search_stop");
         BIND("meta-c", "/input search_switch_case");
-        BIND("ctrl-r", "/input search_switch_regex");
+        BIND("ctrl-x", "/input search_switch_regex");
         BIND("tab",    "/input search_switch_where");
+        BIND("ctrl-r", "/input search_previous");
         BIND("up",     "/input search_previous");
+        BIND("ctrl-s", "/input search_next");
         BIND("down",   "/input search_next");
+        if (context == GUI_KEY_CONTEXT_HISTSEARCH)
+        {
+            BIND("ctrl-o", "/input history_use_get_next");
+        }
     }
     else if (context == GUI_KEY_CONTEXT_CURSOR)
     {
@@ -321,14 +331,7 @@ gui_key_flush (int paste)
         insert_ok = 1;
         utf_partial_char[0] = '\0';
 
-        if (!paste && gui_mouse_event_pending)
-        {
-            insert_ok = 0;
-            key_str[0] = (char)key;
-            key_str[1] = '\0';
-            length_key_str = 1;
-        }
-        else if (!paste && (key < 32))
+        if (!paste && (key < 32))
         {
             insert_ok = 0;
             key_str[0] = '\x01';
@@ -343,6 +346,13 @@ gui_key_flush (int paste)
             key_str[2] = '\0';
             length_key_str = 2;
         }
+        else if (!paste && gui_mouse_event_pending)
+        {
+            insert_ok = 0;
+            key_str[0] = (char)key;
+            key_str[1] = '\0';
+            length_key_str = 1;
+        }
         else if (!paste && (key == 127))
         {
             insert_ok = 0;
@@ -413,7 +423,7 @@ gui_key_flush (int paste)
                 gui_key_last_key_pressed_sent = i;
             }
 
-            if (gui_current_window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED)
+            if (gui_current_window->buffer->text_search != GUI_BUFFER_SEARCH_DISABLED)
                 input_old = (gui_current_window->buffer->input_buffer) ?
                     strdup (gui_current_window->buffer->input_buffer) : strdup ("");
             else
@@ -433,17 +443,17 @@ gui_key_flush (int paste)
                 undo_done = 1;
             }
 
-            /* incremental text search in buffer */
+            /* incremental text search in buffer lines or command line history */
             if ((old_buffer == gui_current_window->buffer)
-                && (gui_current_window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED)
+                && ((gui_current_window->buffer->text_search == GUI_BUFFER_SEARCH_LINES)
+                    || (gui_current_window->buffer->text_search == GUI_BUFFER_SEARCH_HISTORY))
                 && ((input_old == NULL)
                     || (gui_current_window->buffer->input_buffer == NULL)
                     || (strcmp (input_old, gui_current_window->buffer->input_buffer) != 0)))
             {
                 /*
                  * if following conditions are all true, then do not search
-                 * again (search will not find any result and can take some time
-                 * on a buffer with many lines):
+                 * again (search will not find any result and can take some time):
                  * - old search was not successful
                  * - searching a string (not a regex)
                  * - current input is longer than old input
@@ -459,10 +469,7 @@ gui_key_flush (int paste)
                     && (strncmp (gui_current_window->buffer->input_buffer, input_old,
                                  strlen (input_old)) == 0))
                 {
-                    /*
-                     * do not search text in buffer, just alert about text not
-                     * found
-                     */
+                    /* do not search text, just alert about text not found */
                     if (CONFIG_BOOLEAN(config_look_search_text_not_found_alert))
                     {
                         fprintf (stderr, "\a");
@@ -475,8 +482,7 @@ gui_key_flush (int paste)
                 }
             }
 
-            if (input_old)
-                free (input_old);
+            free (input_old);
         }
 
         /* prepare incomplete UTF-8 char for next iteration */
