@@ -511,26 +511,16 @@ irc_message_parse_to_hashtable (struct t_irc_server *server,
     snprintf (str_pos, sizeof (str_pos), "%d", pos_text);
     weechat_hashtable_set (hashtable, "pos_text", str_pos);
 
-    if (tags)
-        free (tags);
-    if (message_without_tags)
-        free (message_without_tags);
-    if (nick)
-        free (nick);
-    if (user)
-        free (user);
-    if (host)
-        free (host);
-    if (command)
-        free (command);
-    if (channel)
-        free (channel);
-    if (arguments)
-        free (arguments);
-    if (text)
-        free (text);
-    if (params)
-        weechat_string_free_split (params);
+    free (tags);
+    free (message_without_tags);
+    free (nick);
+    free (user);
+    free (host);
+    free (command);
+    free (channel);
+    free (arguments);
+    free (text);
+    weechat_string_free_split (params);
 
     return hashtable;
 }
@@ -845,14 +835,10 @@ irc_message_ignored (struct t_irc_server *server, const char *message)
         nick,
         host_no_color);
 
-    if (nick)
-        free (nick);
-    if (host)
-        free (host);
-    if (host_no_color)
-        free (host_no_color);
-    if (channel)
-        free (channel);
+    free (nick);
+    free (host);
+    free (host_no_color);
+    free (channel);
 
     return ignored;
 }
@@ -1295,10 +1281,8 @@ irc_message_split_join (struct t_irc_message_split_context *context,
                                msg_to_send + length_no_channel + 1);
     }
 
-    if (channels)
-        weechat_string_free_split (channels);
-    if (keys)
-        weechat_string_free_split (keys);
+    weechat_string_free_split (channels);
+    weechat_string_free_split (keys);
 
     return 1;
 }
@@ -1683,8 +1667,9 @@ irc_message_split (struct t_irc_server *server, const char *message)
     }
 
     multiline = (
-        ((weechat_strcasecmp (command, "privmsg") == 0)
-         || (weechat_strcasecmp (command, "notice") == 0))
+        server
+        && ((weechat_strcasecmp (command, "privmsg") == 0)
+            || (weechat_strcasecmp (command, "notice") == 0))
         && message
         && strchr (message, '\n')
         && (index_args + 1 <= argc - 1)
@@ -1840,12 +1825,9 @@ end:
                                arguments);
     }
 
-    if (tags)
-        free (tags);
-    if (argv)
-        weechat_string_free_split (argv);
-    if (argv_eol)
-        weechat_string_free_split (argv_eol);
+    free (tags);
+    weechat_string_free_split (argv);
+    weechat_string_free_split (argv_eol);
 
     return split_context.hashtable;
 }
