@@ -123,10 +123,8 @@ irc_raw_message_match_filter (struct t_irc_raw_message *raw_message,
             hashtable,
             irc_raw_filter_hashtable_options);
         match = (result && (strcmp (result, "1") == 0)) ? 1 : 0;
-        if (hashtable)
-            weechat_hashtable_free (hashtable);
-        if (result)
-            free (result);
+        weechat_hashtable_free (hashtable);
+        free (result);
         return match;
     }
     else if (strncmp (filter, "s:", 2) == 0)
@@ -177,8 +175,7 @@ irc_raw_message_match_filter (struct t_irc_raw_message *raw_message,
                            NULL);  /* pos_text */
         match = (command && (weechat_strcasecmp (command, filter + 2) == 0)) ?
             1 : 0;
-        if (command)
-            free (command);
+        free (command);
         return match;
     }
     else
@@ -309,10 +306,8 @@ irc_raw_message_print (struct t_irc_raw_message *raw_message)
         prefix,
         (buf2) ? buf2 : ((buf) ? buf : raw_message->message));
 
-    if (buf)
-        free (buf);
-    if (buf2)
-        free (buf2);
+    free (buf);
+    free (buf2);
 }
 
 /*
@@ -411,8 +406,7 @@ irc_raw_open (int switch_to_buffer)
                 buffer_props,
                 &irc_input_data_cb, NULL, NULL,
                 &irc_buffer_close_cb, NULL, NULL);
-            if (buffer_props)
-                weechat_hashtable_free (buffer_props);
+            weechat_hashtable_free (buffer_props);
 
             /* failed to create buffer ? then return */
             if (!irc_raw_buffer)
@@ -441,8 +435,7 @@ irc_raw_open (int switch_to_buffer)
 void
 irc_raw_set_filter (const char *filter)
 {
-    if (irc_raw_filter)
-        free (irc_raw_filter);
+    free (irc_raw_filter);
     irc_raw_filter = (filter && (strcmp (filter, "*") != 0)) ?
         strdup (filter) : NULL;
     irc_raw_set_localvar_filter ();
@@ -487,8 +480,7 @@ irc_raw_message_free (struct t_irc_raw_message *raw_message)
         (raw_message->next_message)->prev_message = raw_message->prev_message;
 
     /* free data */
-    if (raw_message->message)
-        free (raw_message->message);
+    free (raw_message->message);
 
     free (raw_message);
 
