@@ -1,7 +1,7 @@
 /*
  * logger-command.c - logger commands
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -171,35 +171,36 @@ logger_command_init ()
            " || set <level>"
            " || flush"
            " || disable"),
-        N_("   list: show logging status for opened buffers\n"
-           "    set: set logging level on current buffer\n"
-           "  level: level for messages to be logged (0 = logging disabled, "
-           "1 = a few messages (most important) .. 9 = all messages)\n"
-           "  flush: write all log files now\n"
-           "disable: disable logging on current buffer (set level to 0)\n"
-           "\n"
-           "Options \"logger.level.*\" and \"logger.mask.*\" can be used to set "
-           "level or mask for a buffer, or buffers beginning with name.\n"
-           "\n"
-           "Log levels used by IRC plugin:\n"
-           "  1: user message (channel and private), "
-           "notice (server and channel)\n"
-           "  2: nick change\n"
-           "  3: server message\n"
-           "  4: join/part/quit\n"
-           "  9: all other messages\n"
-           "\n"
-           "Examples:\n"
-           "  set level to 5 for current buffer:\n"
-           "    /logger set 5\n"
-           "  disable logging for current buffer:\n"
-           "    /logger disable\n"
-           "  set level to 3 for all IRC buffers:\n"
-           "    /set logger.level.irc 3\n"
-           "  disable logging for main WeeChat buffer:\n"
-           "    /set logger.level.core.weechat 0\n"
-           "  use a directory per IRC server and a file per channel inside:\n"
-           "    /set logger.mask.irc \"$server/$channel.weechatlog\""),
+        WEECHAT_CMD_ARGS_DESC(
+            N_("raw[list]: show logging status for opened buffers"),
+            N_("raw[set]: set logging level on current buffer"),
+            N_("level: level for messages to be logged (0 = logging disabled, "
+               "1 = a few messages (most important) .. 9 = all messages)"),
+            N_("raw[flush]: write all log files now"),
+            N_("raw[disable]: disable logging on current buffer (set level to 0)"),
+            "",
+            N_("Options \"logger.level.*\" and \"logger.mask.*\" can be used to set "
+               "level or mask for a buffer, or buffers beginning with name."),
+            "",
+            N_("Log levels used by IRC plugin:"),
+            N_("  1: user message (channel and private), "
+               "notice (server and channel)"),
+            N_("  2: nick change"),
+            N_("  3: server message"),
+            N_("  4: join/part/quit"),
+            N_("  9: all other messages"),
+            "",
+            N_("Examples:"),
+            N_("  set level to 5 for current buffer:"),
+            AI("    /logger set 5"),
+            N_("  disable logging for current buffer:"),
+            AI("    /logger disable"),
+            N_("  set level to 3 for all IRC buffers:"),
+            AI("    /set logger.level.irc 3"),
+            N_("  disable logging for main WeeChat buffer:"),
+            AI("    /set logger.level.core.weechat 0"),
+            N_("  use a directory per IRC server and a file per channel inside:"),
+            AI("    /set logger.mask.irc \"$server/$channel.weechatlog\"")),
         "list"
         " || set 1|2|3|4|5|6|7|8|9"
         " || flush"
