@@ -1,5 +1,5 @@
 /* This is a generated file, edit the .stub.php file instead.
- * Stub hash: b6e9e3f12ed24566eb77aa0c08bf3e7c5d866b76 */
+ * Stub hash: cf4a06ff974bca04f671e75e22f7d16534ca643f */
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_weechat_register, 0, 0, 7)
 	ZEND_ARG_INFO(0, p0)
@@ -126,6 +126,10 @@ ZEND_END_ARG_INFO()
 
 #define arginfo_weechat_config_option_rename arginfo_weechat_iconv_to_internal
 
+#define arginfo_weechat_config_option_get_string arginfo_weechat_iconv_to_internal
+
+#define arginfo_weechat_config_option_get_pointer arginfo_weechat_iconv_to_internal
+
 #define arginfo_weechat_config_option_is_null arginfo_weechat_plugin_get_name
 
 #define arginfo_weechat_config_option_default_is_null arginfo_weechat_plugin_get_name
@@ -134,22 +138,32 @@ ZEND_END_ARG_INFO()
 
 #define arginfo_weechat_config_boolean_default arginfo_weechat_plugin_get_name
 
+#define arginfo_weechat_config_boolean_inherited arginfo_weechat_plugin_get_name
+
 #define arginfo_weechat_config_integer arginfo_weechat_plugin_get_name
 
 #define arginfo_weechat_config_integer_default arginfo_weechat_plugin_get_name
 
+#define arginfo_weechat_config_integer_inherited arginfo_weechat_plugin_get_name
+
 #define arginfo_weechat_config_string arginfo_weechat_plugin_get_name
 
 #define arginfo_weechat_config_string_default arginfo_weechat_plugin_get_name
 
+#define arginfo_weechat_config_string_inherited arginfo_weechat_plugin_get_name
+
 #define arginfo_weechat_config_color arginfo_weechat_plugin_get_name
 
 #define arginfo_weechat_config_color_default arginfo_weechat_plugin_get_name
 
+#define arginfo_weechat_config_color_inherited arginfo_weechat_plugin_get_name
+
 #define arginfo_weechat_config_enum arginfo_weechat_plugin_get_name
 
 #define arginfo_weechat_config_enum_default arginfo_weechat_plugin_get_name
 
+#define arginfo_weechat_config_enum_inherited arginfo_weechat_plugin_get_name
+
 #define arginfo_weechat_config_write_option arginfo_weechat_iconv_to_internal
 
 #define arginfo_weechat_config_write_line arginfo_weechat_ngettext
@@ -437,6 +451,8 @@ ZEND_END_ARG_INFO()
 
 #define arginfo_weechat_hdata_long arginfo_weechat_ngettext
 
+#define arginfo_weechat_hdata_longlong arginfo_weechat_ngettext
+
 #define arginfo_weechat_hdata_string arginfo_weechat_ngettext
 
 #define arginfo_weechat_hdata_pointer arginfo_weechat_ngettext
@@ -462,3 +478,4 @@ ZEND_END_ARG_INFO()
 #define arginfo_forget_class arginfo_weechat_plugin_get_name
 
 #define arginfo_forget_function arginfo_weechat_plugin_get_name
+
