@@ -21,6 +21,7 @@
 
 #include <unistd.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <limits.h>
 #include <regex.h>
@@ -31,10 +32,11 @@
 
 #include "../weechat-plugin.h"
 #include "relay.h"
-#include "relay-config.h"
 #include "relay-client.h"
+#include "relay-config.h"
 #include "relay-buffer.h"
 #include "relay-network.h"
+#include "relay-remote.h"
 #include "relay-server.h"
 #include "irc/relay-irc.h"
 
@@ -47,13 +49,15 @@ struct t_config_section *relay_config_section_look = NULL;
 struct t_config_section *relay_config_section_color = NULL;
 struct t_config_section *relay_config_section_network = NULL;
 struct t_config_section *relay_config_section_irc = NULL;
-struct t_config_section *relay_config_section_weechat = NULL;
+struct t_config_section *relay_config_section_api = NULL;
 struct t_config_section *relay_config_section_port = NULL;
 struct t_config_section *relay_config_section_path = NULL;
+struct t_config_section *relay_config_section_remote = NULL;
 
 /* relay config, look section */
 
 struct t_config_option *relay_config_look_auto_open_buffer = NULL;
+struct t_config_option *relay_config_look_display_clients = NULL;
 struct t_config_option *relay_config_look_raw_messages = NULL;
 
 /* relay config, color section */
@@ -73,6 +77,7 @@ struct t_config_option *relay_config_network_allowed_ips = NULL;
 struct t_config_option *relay_config_network_auth_timeout = NULL;
 struct t_config_option *relay_config_network_bind_address = NULL;
 struct t_config_option *relay_config_network_clients_purge_delay = NULL;
+struct t_config_option *relay_config_network_commands = NULL;
 struct t_config_option *relay_config_network_compression = NULL;
 struct t_config_option *relay_config_network_ipv6 = NULL;
 struct t_config_option *relay_config_network_max_clients = NULL;
@@ -80,6 +85,7 @@ struct t_config_option *relay_config_network_nonce_size = NULL;
 struct t_config_option *relay_config_network_password = NULL;
 struct t_config_option *relay_config_network_password_hash_algo = NULL;
 struct t_config_option *relay_config_network_password_hash_iterations = NULL;
+struct t_config_option *relay_config_network_time_window = NULL;
 struct t_config_option *relay_config_network_tls_cert_key = NULL;
 struct t_config_option *relay_config_network_tls_priorities = NULL;
 struct t_config_option *relay_config_network_totp_secret = NULL;
@@ -95,18 +101,109 @@ struct t_config_option *relay_config_irc_backlog_since_last_message = NULL;
 struct t_config_option *relay_config_irc_backlog_tags = NULL;
 struct t_config_option *relay_config_irc_backlog_time_format = NULL;
 
-/* relay config, weechat section */
+/* relay config, api section */
 
-struct t_config_option *relay_config_weechat_commands = NULL;
+struct t_config_option *relay_config_api_remote_get_lines = NULL;
 
 /* other */
 
+int relay_config_auto_open_buffer[RELAY_NUM_PROTOCOLS];
+int relay_config_display_clients[RELAY_NUM_PROTOCOLS];
 regex_t *relay_config_regex_allowed_ips = NULL;
 regex_t *relay_config_regex_websocket_allowed_origins = NULL;
 struct t_hashtable *relay_config_hashtable_irc_backlog_tags = NULL;
 char **relay_config_network_password_hash_algo_list = NULL;
 
 
+/*
+ * Callback for changes on option "relay.look.auto_open_buffer".
+ */
+
+void
+relay_config_change_auto_open_buffer_cb (const void *pointer, void *data,
+                                         struct t_config_option *option)
+{
+    const char *auto_open_buffer;
+    const char *value_on = "irc,weechat", *value_off = "";
+    char **items;
+    int i, num_items, protocol;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) option;
+
+    auto_open_buffer = weechat_config_string (relay_config_look_auto_open_buffer);
+
+    /* old option was a boolean, use these values for compatibility */
+    if (strcmp (auto_open_buffer, "on") == 0)
+        auto_open_buffer = value_on;
+    else if (strcmp (auto_open_buffer, "off") == 0)
+        auto_open_buffer = value_off;
+
+    for (i = 0; i < RELAY_NUM_PROTOCOLS; i++)
+    {
+        relay_config_auto_open_buffer[i] = 0;
+    }
+
+    if (auto_open_buffer[0])
+    {
+        items = weechat_string_split (auto_open_buffer, ",", NULL, 0, 0,
+                                      &num_items);
+        if (items)
+        {
+            for (i = 0; i < num_items; i++)
+            {
+                protocol = relay_protocol_search (items[i]);
+                if (protocol >= 0)
+                    relay_config_auto_open_buffer[protocol] = 1;
+            }
+            weechat_string_free_split (items);
+        }
+    }
+}
+
+/*
+ * Callback for changes on option "relay.look.display_clients".
+ */
+
+void
+relay_config_change_display_clients_cb (const void *pointer, void *data,
+                                        struct t_config_option *option)
+{
+    const char *display_clients;
+    char **items;
+    int i, num_items, protocol;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) option;
+
+    display_clients = weechat_config_string (relay_config_look_display_clients);
+
+    for (i = 0; i < RELAY_NUM_PROTOCOLS; i++)
+    {
+        relay_config_display_clients[i] = 0;
+    }
+
+    if (display_clients[0])
+    {
+        items = weechat_string_split (display_clients, ",", NULL, 0, 0,
+                                      &num_items);
+        if (items)
+        {
+            for (i = 0; i < num_items; i++)
+            {
+                protocol = relay_protocol_search (items[i]);
+                if (protocol >= 0)
+                    relay_config_display_clients[protocol] = 1;
+            }
+            weechat_string_free_split (items);
+        }
+    }
+}
+
 /*
  * Callback for changes on options that require a refresh of relay list.
  */
@@ -290,7 +387,7 @@ relay_config_check_network_totp_secret (const void *pointer, void *data,
         secret = malloc (strlen (totp_secret) + 1);
         if (!secret)
             goto error;
-        length = weechat_string_base_decode (32, totp_secret, secret);
+        length = weechat_string_base_decode ("32", totp_secret, secret);
         if (length < 0)
             goto error;
     }
@@ -308,10 +405,8 @@ error:
                     weechat_prefix ("error"), RELAY_PLUGIN_NAME);
 
 end:
-    if (totp_secret)
-        free (totp_secret);
-    if (secret)
-        free (secret);
+    free (totp_secret);
+    free (secret);
     return rc;
 }
 
@@ -687,8 +782,7 @@ relay_config_delete_path_cb (const void *pointer, void *data,
     (void) data;
 
     ptr_server = relay_server_search (weechat_config_option_get_pointer (option, "name"));
-    if (ptr_server)
-        relay_server_free (ptr_server);
+    relay_server_free (ptr_server);
 }
 
 /*
@@ -728,8 +822,7 @@ relay_config_delete_port_cb (const void *pointer, void *data,
     (void) data;
 
     ptr_server = relay_server_search (weechat_config_option_get_pointer (option, "name"));
-    if (ptr_server)
-        relay_server_free (ptr_server);
+    relay_server_free (ptr_server);
 }
 
 /*
@@ -862,14 +955,320 @@ relay_config_create_option_port_path (const void *pointer, void *data,
             rc = WEECHAT_CONFIG_OPTION_SET_ERROR;
     }
 
-    if (protocol)
-        free (protocol);
-    if (protocol_args)
-        free (protocol_args);
+    free (protocol);
+    free (protocol_args);
 
     return rc;
 }
 
+/*
+ * Gets remote pointer with name of option.
+ */
+
+struct t_relay_remote *
+relay_config_get_remote_from_option_name (const char *name)
+{
+    struct t_relay_remote *ptr_remote;
+    char *pos_option, *remote_name;
+
+    ptr_remote = NULL;
+
+    if (name)
+    {
+        pos_option = strrchr (name, '.');
+        if (pos_option)
+        {
+            remote_name = weechat_strndup (name, pos_option - name);
+            if (remote_name)
+            {
+                ptr_remote = relay_remote_search (remote_name);
+                free (remote_name);
+            }
+        }
+    }
+
+    return ptr_remote;
+}
+
+/*
+ * Callback called to check a server option when it is modified.
+ */
+
+int
+relay_config_remote_url_check_value_cb (const void *pointer, void *data,
+                                        struct t_config_option *option,
+                                        const char *value)
+{
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) option;
+
+    return relay_remote_url_valid (value);
+}
+
+/*
+ * Callback called when a remote URL option is modified.
+ */
+
+void
+relay_config_remote_url_change_cb (const void *pointer, void *data,
+                                   struct t_config_option *option)
+{
+    struct t_relay_remote *ptr_remote;
+    char *name;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+
+    name = weechat_config_option_get_pointer (option, "name");
+    ptr_remote = relay_config_get_remote_from_option_name (name);
+    if (ptr_remote)
+        relay_remote_set_url (ptr_remote, weechat_config_string (option));
+}
+
+/*
+ * Creates an option for a remote.
+ *
+ * Returns pointer to new option, NULL if error.
+ */
+
+struct t_config_option *
+relay_config_create_remote_option (const char *remote_name, int index_option,
+                                   const char *value)
+{
+    struct t_config_option *ptr_option;
+    int length;
+    char *option_name;
+
+    ptr_option = NULL;
+
+    length = strlen (remote_name) + 1 +
+        strlen (relay_remote_option_string[index_option]) + 1;
+    option_name = malloc (length);
+    if (!option_name)
+        return NULL;
+
+    snprintf (option_name, length, "%s.%s",
+              remote_name, relay_remote_option_string[index_option]);
+
+    switch (index_option)
+    {
+        case RELAY_REMOTE_OPTION_URL:
+            ptr_option = weechat_config_new_option (
+                relay_config_file, relay_config_section_remote,
+                option_name, "string",
+                N_("remote relay URL with optional port (default is 9000), "
+                   "examples: https://example.com:9000 or http://example.com:9000 "
+                   "(plain-text connection, not recommended)"),
+                NULL, 0, 0, value, NULL, 0,
+                &relay_config_remote_url_check_value_cb, NULL, NULL,
+                &relay_config_remote_url_change_cb, NULL, NULL,
+                NULL, NULL, NULL);
+            break;
+        case RELAY_REMOTE_OPTION_AUTOCONNECT:
+            ptr_option = weechat_config_new_option (
+                relay_config_file, relay_config_section_remote,
+                option_name, "boolean",
+                N_("auto-connect to the remote relay"),
+                NULL, 0, 0, value, NULL, 0,
+                NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+            break;
+        case RELAY_REMOTE_OPTION_PROXY:
+            ptr_option = weechat_config_new_option (
+                relay_config_file, relay_config_section_remote,
+                option_name, "string",
+                N_("name of proxy used for this remote relay (optional, proxy "
+                   "must be defined with command /proxy)"),
+                NULL, 0, 0, value, NULL, 0,
+                NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+            break;
+        case RELAY_REMOTE_OPTION_TLS_VERIFY:
+            ptr_option = weechat_config_new_option (
+                relay_config_file, relay_config_section_remote,
+                option_name, "boolean",
+                N_("check that the TLS connection is fully trusted"),
+                NULL, 0, 0, value, NULL, 0,
+                NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+            break;
+        case RELAY_REMOTE_OPTION_PASSWORD:
+            ptr_option = weechat_config_new_option (
+                relay_config_file, relay_config_section_remote,
+                option_name, "string",
+                N_("password for remote relay "
+                   "(note: content is evaluated, see /help eval)"),
+                NULL, 0, 0, value, NULL, 0,
+                NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+            break;
+        case RELAY_REMOTE_OPTION_TOTP_SECRET:
+            ptr_option = weechat_config_new_option (
+                relay_config_file, relay_config_section_remote,
+                option_name, "string",
+                N_("TOTP secret, encoded in base32 "
+                   "(note: content is evaluated, see /help eval)"),
+                NULL, 0, 0, value, NULL, 0,
+                NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+            break;
+        case RELAY_REMOTE_NUM_OPTIONS:
+            break;
+    }
+
+    free (option_name);
+
+    return ptr_option;
+}
+
+/*
+ * Creates option for a temporary remote (when reading configuration file).
+ */
+
+void
+relay_config_create_option_temp (struct t_relay_remote *temp_remote,
+                                 int index_option, const char *value)
+{
+    struct t_config_option *new_option;
+
+    new_option = relay_config_create_remote_option (temp_remote->name,
+                                                    index_option, value);
+    if (new_option
+        && (index_option >= 0) && (index_option < RELAY_REMOTE_NUM_OPTIONS))
+    {
+        temp_remote->options[index_option] = new_option;
+    }
+}
+
+/*
+ * Uses temporary remotes (created by reading configuration file).
+ */
+
+void
+relay_config_use_temp_remotes ()
+{
+    struct t_relay_remote *ptr_temp_remote, *next_temp_remote;
+    int i, num_options_ok;
+
+    for (ptr_temp_remote = relay_remotes_temp; ptr_temp_remote;
+         ptr_temp_remote = ptr_temp_remote->next_remote)
+    {
+        num_options_ok = 0;
+        for (i = 0; i < RELAY_REMOTE_NUM_OPTIONS; i++)
+        {
+            if (!ptr_temp_remote->options[i])
+            {
+                ptr_temp_remote->options[i] =
+                    relay_config_create_remote_option (
+                        ptr_temp_remote->name,
+                        i,
+                        relay_remote_option_default[i]);
+            }
+            if (ptr_temp_remote->options[i])
+                num_options_ok++;
+        }
+
+        if (num_options_ok == RELAY_REMOTE_NUM_OPTIONS)
+        {
+            relay_remote_new_with_options (ptr_temp_remote->name,
+                                           ptr_temp_remote->options);
+        }
+        else
+        {
+            for (i = 0; i < RELAY_REMOTE_NUM_OPTIONS; i++)
+            {
+                if (ptr_temp_remote->options[i])
+                {
+                    weechat_config_option_free (ptr_temp_remote->options[i]);
+                    ptr_temp_remote->options[i] = NULL;
+                }
+            }
+        }
+    }
+
+    /* free all temporary remotes */
+    while (relay_remotes_temp)
+    {
+        next_temp_remote = relay_remotes_temp->next_remote;
+
+        free (relay_remotes_temp->name);
+        free (relay_remotes_temp);
+
+        relay_remotes_temp = next_temp_remote;
+    }
+    last_relay_remote_temp = NULL;
+}
+
+/*
+ * Reads a remote option in relay configuration file.
+ */
+
+int
+relay_config_remote_read_cb (const void *pointer, void *data,
+                             struct t_config_file *config_file,
+                             struct t_config_section *section,
+                             const char *option_name, const char *value)
+{
+    char *pos_option, *remote_name;
+    struct t_relay_remote *ptr_temp_remote;
+    int index_option;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) config_file;
+    (void) section;
+
+    if (!option_name)
+        return WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE;
+
+    pos_option = strchr (option_name, '.');
+    if (!pos_option)
+        return WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE;
+
+    remote_name = weechat_strndup (option_name, pos_option - option_name);
+    if (!remote_name)
+        return WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE;
+
+    pos_option++;
+
+    /* search temporary remote */
+    for (ptr_temp_remote = relay_remotes_temp; ptr_temp_remote;
+         ptr_temp_remote = ptr_temp_remote->next_remote)
+    {
+        if (strcmp (ptr_temp_remote->name, remote_name) == 0)
+            break;
+    }
+    if (!ptr_temp_remote)
+    {
+        /* create new temporary remote */
+        ptr_temp_remote = relay_remote_alloc (remote_name);
+        if (ptr_temp_remote)
+            relay_remote_add (ptr_temp_remote, &relay_remotes_temp, &last_relay_remote_temp);
+    }
+
+    if (ptr_temp_remote)
+    {
+        index_option = relay_remote_search_option (pos_option);
+        if (index_option >= 0)
+        {
+            relay_config_create_option_temp (ptr_temp_remote, index_option,
+                                             value);
+        }
+        else
+        {
+            weechat_printf (NULL,
+                            _("%sWarning: unknown option for section \"%s\": "
+                              "%s (value: \"%s\")"),
+                            weechat_prefix ("error"),
+                            RELAY_CONFIG_SECTION_REMOTE,
+                            option_name, value);
+        }
+    }
+
+    free (remote_name);
+
+    return WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE;
+}
+
 /*
  * Reloads relay configuration file.
  */
@@ -916,7 +1315,7 @@ relay_config_update_cb (const void *pointer, void *data,
     if (version_read < 2)
     {
         /*
-         * changes in v2:
+         * changes in v2 (WeeChat 4.0.0):
          *   - options "ssl*" renamed to "tls*"
          *   - protocol "ssl" renamed to "tls" in port/path sections
          */
@@ -1030,10 +1429,24 @@ relay_config_init ()
     {
         relay_config_look_auto_open_buffer = weechat_config_new_option (
             relay_config_file, relay_config_section_look,
-            "auto_open_buffer", "boolean",
-            N_("auto open relay buffer when a new client is connecting"),
-            NULL, 0, 0, "on", NULL, 0,
-            NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+            "auto_open_buffer", "string",
+            N_("auto open relay buffer when a new client is connecting "
+               "using one of these protocols (comma-separated list); "
+               "allowed protocols: \"irc\", \"weechat\", \"api\""),
+            NULL, 0, 0, "irc,weechat", NULL, 0,
+            NULL, NULL, NULL,
+            &relay_config_change_auto_open_buffer_cb, NULL, NULL,
+            NULL, NULL, NULL);
+        relay_config_look_display_clients = weechat_config_new_option (
+            relay_config_file, relay_config_section_look,
+            "display_clients", "string",
+            N_("display messages when clients connect/disconnect from relay "
+               "using one of these protocols (comma-separated list); "
+               "allowed protocols: \"irc\", \"weechat\", \"api\""),
+            NULL, 0, 0, "irc,weechat", NULL, 0,
+            NULL, NULL, NULL,
+            &relay_config_change_display_clients_cb, NULL, NULL,
+            NULL, NULL, NULL);
         relay_config_look_raw_messages = weechat_config_new_option (
             relay_config_file, relay_config_section_look,
             "raw_messages", "integer",
@@ -1093,10 +1506,10 @@ relay_config_init ()
             NULL, NULL, NULL,
             &relay_config_refresh_cb, NULL, NULL,
             NULL, NULL, NULL);
-        relay_config_color_status[RELAY_STATUS_WAITING_AUTH] = weechat_config_new_option (
+        relay_config_color_status[RELAY_STATUS_AUTHENTICATING] = weechat_config_new_option (
             relay_config_file, relay_config_section_color,
-            "status_waiting_auth", "color",
-            N_("text color for \"waiting authentication\" status"),
+            "status_authenticating", "color",
+            N_("text color for \"authenticating\" status"),
             NULL, 0, 0, "yellow", NULL, 0,
             NULL, NULL, NULL,
             &relay_config_refresh_cb, NULL, NULL,
@@ -1182,14 +1595,30 @@ relay_config_init ()
                "clients immediately, -1 = never purge)"),
             NULL, -1, 60 * 24 * 30, "0", NULL, 0,
             NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+        relay_config_network_commands = weechat_config_new_option (
+            relay_config_file, relay_config_section_network,
+            "commands", "string",
+            N_("comma-separated list of commands allowed/denied when input "
+               "data (text or command) is received from a client (weechat "
+               "and api protocols); "
+               "\"*\" means any command, a name beginning with \"!\" is "
+               "a negative value to prevent a command from being executed, "
+               "wildcard \"*\" is allowed in names; this option should be set "
+               "if the relay client is not safe (someone could use it to run "
+               "commands); for example \"*,!exec,!quit\" allows any command "
+               "except /exec and /quit"),
+            NULL, 0, 0, "*,!quit", NULL, 0,
+            NULL, NULL, NULL,
+            NULL, NULL, NULL,
+            NULL, NULL, NULL);
         relay_config_network_compression = weechat_config_new_option (
             relay_config_file, relay_config_section_network,
             "compression", "integer",
-            N_("compression of messages sent to clients with \"weechat\" "
-               "protocol: 0 = disable compression, 1 = low compression / fast "
-               "... 100 = best compression / slow; the value is a percentage "
-               "converted to 1-9 for zlib and 1-19 for zstd; "
-               "the default value is recommended, it offers a good "
+            N_("compression of messages sent to clients with weechat and "
+               "api protocols: 0 = disable compression, "
+               "1 = low compression / fast ... 100 = best compression / slow; "
+               "the value is a percentage converted to 1-9 for zlib and 1-19 "
+               "for zstd; the default value is recommended, it offers a good "
                "compromise between compression and speed"),
             NULL, 0, 100, "20", NULL, 0,
             NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
@@ -1252,6 +1681,13 @@ relay_config_init ()
                "if your CPU is slow"),
             NULL, 1, 1000000, "100000", NULL, 0,
             NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+        relay_config_network_time_window = weechat_config_new_option (
+            relay_config_file, relay_config_section_network,
+            "time_window", "integer",
+            N_("number of seconds to allow before and after the current time "
+               "for the hash of time + password in api protocol"),
+            NULL, 0, 256, "5", NULL, 0,
+            NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
         relay_config_network_tls_cert_key = weechat_config_new_option (
             relay_config_file, relay_config_section_network,
             "tls_cert_key", "string",
@@ -1279,8 +1715,8 @@ relay_config_init ()
             "totp_secret", "string",
             N_("secret for the generation of the Time-based One-Time Password "
                "(TOTP), encoded in base32 (only letters and digits from 2 to 7); "
-               "it is used as second factor in weechat protocol, in addition to "
-               "the password, which must not be empty "
+               "it is used as second factor in weechat and api protocols, "
+               "in addition to the password, which must not be empty "
                "(empty value means no TOTP is required) "
                "(note: content is evaluated, see /help eval)"),
             NULL, 0, 0, "", NULL, 0,
@@ -1372,32 +1808,24 @@ relay_config_init ()
             NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
     }
 
-    /* section weechat */
-    relay_config_section_weechat = weechat_config_new_section (
-        relay_config_file, "weechat",
+    /* section api */
+    relay_config_section_api = weechat_config_new_section (
+        relay_config_file, "api",
         0, 0,
         NULL, NULL, NULL,
         NULL, NULL, NULL,
         NULL, NULL, NULL,
         NULL, NULL, NULL,
         NULL, NULL, NULL);
-    if (relay_config_section_weechat)
+    if (relay_config_section_api)
     {
-        relay_config_weechat_commands = weechat_config_new_option (
-            relay_config_file, relay_config_section_weechat,
-            "commands", "string",
-            N_("comma-separated list of commands allowed/denied when input "
-               "data (text or command) is received from a client; "
-               "\"*\" means any command, a name beginning with \"!\" is "
-               "a negative value to prevent a command from being executed, "
-               "wildcard \"*\" is allowed in names; this option should be set "
-               "if the relay client is not safe (someone could use it to run "
-               "commands); for example \"*,!exec,!quit\" allows any command "
-               "except /exec and /quit"),
-            NULL, 0, 0, "", NULL, 0,
-            NULL, NULL, NULL,
-            NULL, NULL, NULL,
-            NULL, NULL, NULL);
+        relay_config_api_remote_get_lines = weechat_config_new_option (
+            relay_config_file, relay_config_section_api,
+            "remote_get_lines", "integer",
+            N_("number of lines to retrieve on each buffer when connecting "
+               "to a remote relay"),
+            NULL, 0, INT_MAX, "1000", NULL, 0,
+            NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
     }
 
     /* section port */
@@ -1420,6 +1848,17 @@ relay_config_init ()
         &relay_config_create_option_port_path, NULL, NULL,
         NULL, NULL, NULL);
 
+    /* remote */
+    relay_config_section_remote = weechat_config_new_section (
+        relay_config_file,
+        RELAY_CONFIG_SECTION_REMOTE,
+        0, 0,
+        &relay_config_remote_read_cb, NULL, NULL,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL);
+
     return 1;
 }
 
@@ -1435,9 +1874,12 @@ relay_config_read ()
     rc = weechat_config_read (relay_config_file);
     if (rc == WEECHAT_CONFIG_READ_OK)
     {
+        relay_config_change_auto_open_buffer_cb (NULL, NULL, NULL);
+        relay_config_change_display_clients_cb (NULL, NULL, NULL);
         relay_config_change_network_allowed_ips (NULL, NULL, NULL);
         relay_config_change_network_password_hash_algo (NULL, NULL, NULL);
         relay_config_change_irc_backlog_tags (NULL, NULL, NULL);
+        relay_config_use_temp_remotes ();
     }
     return rc;
 }
