@@ -1,7 +1,7 @@
 /*
  * relay-weechat-msg.c - build binary messages for WeeChat protocol
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -175,6 +175,23 @@ relay_weechat_msg_add_long (struct t_relay_weechat_msg *msg, long value)
     relay_weechat_msg_add_bytes (msg, str_long, length);
 }
 
+/*
+ * Adds a long long integer to a message.
+ */
+
+void
+relay_weechat_msg_add_longlong (struct t_relay_weechat_msg *msg,
+                                long long value)
+{
+    char str_longlong[128];
+    unsigned char length;
+
+    snprintf (str_longlong, sizeof (str_longlong), "%lld", value);
+    length = strlen (str_longlong);
+    relay_weechat_msg_add_bytes (msg, &length, 1);
+    relay_weechat_msg_add_bytes (msg, str_longlong, length);
+}
+
 /*
  * Adds length + string to a message.
  */
@@ -434,6 +451,7 @@ relay_weechat_msg_add_hdata_path (struct t_relay_weechat_msg *msg,
                                 relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_INT);
                                 break;
                             case WEECHAT_HDATA_LONG:
+                            case WEECHAT_HDATA_LONGLONG:
                                 relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_LONG);
                                 break;
                             case WEECHAT_HDATA_STRING:
@@ -480,6 +498,12 @@ relay_weechat_msg_add_hdata_path (struct t_relay_weechat_msg *msg,
                                                                                     pointer,
                                                                                     name));
                                     break;
+                                case WEECHAT_HDATA_LONGLONG:
+                                    relay_weechat_msg_add_longlong (msg,
+                                                                    weechat_hdata_longlong (hdata,
+                                                                                            pointer,
+                                                                                            name));
+                                    break;
                                 case WEECHAT_HDATA_STRING:
                                 case WEECHAT_HDATA_SHARED_STRING:
                                     relay_weechat_msg_add_string (msg,
@@ -694,6 +718,7 @@ relay_weechat_msg_add_hdata (struct t_relay_weechat_msg *msg,
                         strcat (keys_types, RELAY_WEECHAT_MSG_OBJ_INT);
                         break;
                     case WEECHAT_HDATA_LONG:
+                    case WEECHAT_HDATA_LONGLONG:
                         strcat (keys_types, RELAY_WEECHAT_MSG_OBJ_LONG);
                         break;
                     case WEECHAT_HDATA_STRING:
@@ -743,16 +768,11 @@ relay_weechat_msg_add_hdata (struct t_relay_weechat_msg *msg,
     rc = 1;
 
 end:
-    if (list_keys)
-        weechat_string_free_split (list_keys);
-    if (keys_types)
-        free (keys_types);
-    if (list_path)
-        weechat_string_free_split (list_path);
-    if (path_returned)
-        free (path_returned);
-    if (hdata_head)
-        free (hdata_head);
+    weechat_string_free_split (list_keys);
+    free (keys_types);
+    weechat_string_free_split (list_path);
+    free (path_returned);
+    free (hdata_head);
 
     return rc;
 }
@@ -882,7 +902,6 @@ relay_weechat_msg_add_nicklist_buffer (struct t_relay_weechat_msg *msg,
                                        struct t_relay_weechat_nicklist *nicklist)
 {
     int count, i;
-    struct t_hdata *ptr_hdata_group, *ptr_hdata_nick;
     struct t_gui_nick_group *ptr_group;
     struct t_gui_nick *ptr_nick;
 
@@ -909,9 +928,6 @@ relay_weechat_msg_add_nicklist_buffer (struct t_relay_weechat_msg *msg,
     else
     {
         /* send full nicklist */
-        ptr_hdata_group = weechat_hdata_get ("nick_group");
-        ptr_hdata_nick = weechat_hdata_get ("nick");
-
         ptr_group = NULL;
         ptr_nick = NULL;
         weechat_nicklist_get_next_item (buffer, &ptr_group, &ptr_nick);
@@ -922,27 +938,32 @@ relay_weechat_msg_add_nicklist_buffer (struct t_relay_weechat_msg *msg,
                 relay_weechat_msg_add_pointer (msg, buffer);
                 relay_weechat_msg_add_pointer (msg, ptr_nick);
                 relay_weechat_msg_add_char (msg, 0); /* group */
-                relay_weechat_msg_add_char (msg,
-                                            (char)weechat_hdata_integer (ptr_hdata_nick,
-                                                                         ptr_nick,
-                                                                         "visible"));
+                relay_weechat_msg_add_char (
+                    msg,
+                    (char)weechat_hdata_integer (relay_hdata_nick,
+                                                 ptr_nick,
+                                                 "visible"));
                 relay_weechat_msg_add_int (msg, 0); /* level */
-                relay_weechat_msg_add_string (msg,
-                                              weechat_hdata_string (ptr_hdata_nick,
-                                                                    ptr_nick,
-                                                                    "name"));
-                relay_weechat_msg_add_string (msg,
-                                              weechat_hdata_string (ptr_hdata_nick,
-                                                                    ptr_nick,
-                                                                    "color"));
-                relay_weechat_msg_add_string (msg,
-                                              weechat_hdata_string (ptr_hdata_nick,
-                                                                    ptr_nick,
-                                                                    "prefix"));
-                relay_weechat_msg_add_string (msg,
-                                              weechat_hdata_string (ptr_hdata_nick,
-                                                                    ptr_nick,
-                                                                    "prefix_color"));
+                relay_weechat_msg_add_string (
+                    msg,
+                    weechat_hdata_string (relay_hdata_nick,
+                                          ptr_nick,
+                                          "name"));
+                relay_weechat_msg_add_string (
+                    msg,
+                    weechat_hdata_string (relay_hdata_nick,
+                                          ptr_nick,
+                                          "color"));
+                relay_weechat_msg_add_string (
+                    msg,
+                    weechat_hdata_string (relay_hdata_nick,
+                                          ptr_nick,
+                                          "prefix"));
+                relay_weechat_msg_add_string (
+                    msg,
+                    weechat_hdata_string (relay_hdata_nick,
+                                          ptr_nick,
+                                          "prefix_color"));
                 count++;
             }
             else
@@ -950,22 +971,26 @@ relay_weechat_msg_add_nicklist_buffer (struct t_relay_weechat_msg *msg,
                 relay_weechat_msg_add_pointer (msg, buffer);
                 relay_weechat_msg_add_pointer (msg, ptr_group);
                 relay_weechat_msg_add_char (msg, 1); /* group */
-                relay_weechat_msg_add_char (msg,
-                                            (char)weechat_hdata_integer (ptr_hdata_group,
-                                                                         ptr_group,
-                                                                         "visible"));
-                relay_weechat_msg_add_int (msg,
-                                           weechat_hdata_integer (ptr_hdata_group,
-                                                                  ptr_group,
-                                                                  "level"));
-                relay_weechat_msg_add_string (msg,
-                                              weechat_hdata_string (ptr_hdata_group,
-                                                                    ptr_group,
-                                                                    "name"));
-                relay_weechat_msg_add_string (msg,
-                                              weechat_hdata_string (ptr_hdata_group,
-                                                                    ptr_group,
-                                                                    "color"));
+                relay_weechat_msg_add_char (
+                    msg,
+                    (char)weechat_hdata_integer (relay_hdata_nick_group,
+                                                 ptr_group,
+                                                 "visible"));
+                relay_weechat_msg_add_int (
+                    msg,
+                    weechat_hdata_integer (relay_hdata_nick_group,
+                                           ptr_group,
+                                           "level"));
+                relay_weechat_msg_add_string (
+                    msg,
+                    weechat_hdata_string (relay_hdata_nick_group,
+                                          ptr_group,
+                                          "name"));
+                relay_weechat_msg_add_string (
+                    msg,
+                    weechat_hdata_string (relay_hdata_nick_group,
+                                          ptr_group,
+                                          "color"));
                 relay_weechat_msg_add_string (msg, NULL); /* prefix */
                 relay_weechat_msg_add_string (msg, NULL); /* prefix_color */
                 count++;
@@ -990,7 +1015,6 @@ relay_weechat_msg_add_nicklist (struct t_relay_weechat_msg *msg,
                                 struct t_relay_weechat_nicklist *nicklist)
 {
     char str_vars[512];
-    struct t_hdata *ptr_hdata;
     struct t_gui_buffer *ptr_buffer;
     int pos_count, count;
     uint32_t count32;
@@ -1016,12 +1040,11 @@ relay_weechat_msg_add_nicklist (struct t_relay_weechat_msg *msg,
     }
     else
     {
-        ptr_hdata = weechat_hdata_get ("buffer");
-        ptr_buffer = weechat_hdata_get_list (ptr_hdata, "gui_buffers");
+        ptr_buffer = weechat_hdata_get_list (relay_hdata_buffer, "gui_buffers");
         while (ptr_buffer)
         {
             count += relay_weechat_msg_add_nicklist_buffer (msg, ptr_buffer, NULL);
-            ptr_buffer = weechat_hdata_move (ptr_hdata, ptr_buffer, 1);
+            ptr_buffer = weechat_hdata_move (relay_hdata_buffer, ptr_buffer, 1);
         }
     }
 
@@ -1088,15 +1111,14 @@ relay_weechat_msg_compress_zlib (struct t_relay_client *client,
               msg->id);
 
     /* send compressed data */
-    relay_client_send (client, RELAY_CLIENT_MSG_STANDARD,
+    relay_client_send (client, RELAY_MSG_STANDARD,
                        (const char *)dest, dest_size + 5,
                        raw_message);
 
     rc = 1;
 
 error:
-    if (dest)
-        free (dest);
+    free (dest);
 
     return rc;
 }
@@ -1135,7 +1157,7 @@ relay_weechat_msg_compress_zstd (struct t_relay_client *client,
     compression_level = (((compression - 1) * 19) / 100) + 1;
 
     gettimeofday (&tv1, NULL);
-    comp_size = ZSTD_compress(
+    comp_size = ZSTD_compress (
         dest + 5,
         dest_size,
         (void *)(msg->data + 5),
@@ -1161,15 +1183,14 @@ relay_weechat_msg_compress_zstd (struct t_relay_client *client,
               msg->id);
 
     /* send compressed data */
-    relay_client_send (client, RELAY_CLIENT_MSG_STANDARD,
+    relay_client_send (client, RELAY_MSG_STANDARD,
                        (const char *)dest, comp_size + 5,
                        raw_message);
 
     rc = 1;
 
 error:
-    if (dest)
-        free (dest);
+    free (dest);
 
     return rc;
 #else
@@ -1222,7 +1243,7 @@ relay_weechat_msg_send (struct t_relay_client *client,
     /* send uncompressed data */
     snprintf (raw_message, sizeof (raw_message),
               "obj: %d bytes, id: %s", msg->data_size, msg->id);
-    relay_client_send (client, RELAY_CLIENT_MSG_STANDARD,
+    relay_client_send (client, RELAY_MSG_STANDARD,
                        msg->data, msg->data_size, raw_message);
 }
 
@@ -1236,10 +1257,8 @@ relay_weechat_msg_free (struct t_relay_weechat_msg *msg)
     if (!msg)
         return;
 
-    if (msg->id)
-        free (msg->id);
-    if (msg->data)
-        free (msg->data);
+    free (msg->id);
+    free (msg->data);
 
     free (msg);
 }
