@@ -2,7 +2,7 @@
  * spell.c - spell checker plugin for WeeChat
  *
  * Copyright (C) 2006 Emmanuel Bouthenot <kolter@openics.org>
- * Copyright (C) 2006-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2006-2024 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2012 Nils Görs <weechatter@arcor.de>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -216,10 +216,8 @@ spell_warning_aspell_config ()
                         spell_filename);
     }
 
-    if (aspell_filename)
-        free (aspell_filename);
-    if (spell_filename)
-        free (spell_filename);
+    free (aspell_filename);
+    free (spell_filename);
 }
 
 /*
@@ -938,8 +936,7 @@ spell_modifier_cb (const void *pointer, void *data,
                      * the beginning of this word, save the word (we will
                      * look for suggestions after this loop)
                      */
-                    if (misspelled_word)
-                        free (misspelled_word);
+                    free (misspelled_word);
                     misspelled_word = strdup (ptr_string);
                 }
             }
@@ -1021,9 +1018,7 @@ spell_modifier_cb (const void *pointer, void *data,
             }
         }
 
-        if (old_misspelled_word)
-            free (old_misspelled_word);
-
+        free (old_misspelled_word);
         free (misspelled_word);
     }
     else
@@ -1046,8 +1041,7 @@ spell_modifier_cb (const void *pointer, void *data,
         (void) weechat_hook_signal_send ("spell_suggest",
                                          WEECHAT_HOOK_SIGNAL_POINTER, buffer);
     }
-    if (old_suggestions)
-        free (old_suggestions);
+    free (old_suggestions);
 
     ptr_speller_buffer->modifier_result = strdup (*result);
 
@@ -1165,8 +1159,7 @@ spell_config_change_nick_completer_cb (const void *pointer, void *data,
     (void) data;
     (void) option;
 
-    if (spell_nick_completer)
-        free (spell_nick_completer);
+    free (spell_nick_completer);
 
     spell_nick_completer = weechat_string_strip (value, 0, 1, " ");
     spell_len_nick_completer =
@@ -1188,6 +1181,8 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 
     weechat_plugin = plugin;
 
+    spell_enabled = 0;
+
     spell_warning_aspell_config ();
 
 #ifdef USE_ENCHANT
@@ -1260,10 +1255,15 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
 #ifdef USE_ENCHANT
     /* release enchant broker */
     enchant_broker_free (broker);
+    broker = NULL;
 #endif /* USE_ENCHANT */
 
     if (spell_nick_completer)
+    {
         free (spell_nick_completer);
+        spell_nick_completer = NULL;
+    }
+    spell_len_nick_completer = 0;
 
     return WEECHAT_RC_OK;
 }
