@@ -1,7 +1,7 @@
 /*
  * xfer-command.c - xfer command
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -258,17 +258,19 @@ xfer_command_init ()
         N_("send a CTCP action to remote host"),
         /* TRANSLATORS: only text between angle brackets (eg: "<name>") must be translated */
         N_("<message>"),
-        N_("message: message to send"),
+        WEECHAT_CMD_ARGS_DESC(
+            N_("message: message to send")),
         NULL,
         &xfer_command_me, NULL, NULL);
     weechat_hook_command (
         "xfer",
         N_("xfer control"),
         "[list|listfull]",
-        N_("    list: list xfer\n"
-           "listfull: list xfer (verbose)\n"
-           "\n"
-           "Without argument, this command opens buffer with xfer list."),
+        WEECHAT_CMD_ARGS_DESC(
+            N_("raw[list]: list xfer"),
+            N_("raw[listfull]: list xfer (verbose)"),
+            "",
+            N_("Without argument, this command opens buffer with xfer list.")),
         "list|listfull",
         &xfer_command_xfer, NULL, NULL);
 }
