@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2017-2023 Sébastien Helleu <flashcode@flashtux.org>
+# Copyright (C) 2017-2024 Sébastien Helleu <flashcode@flashtux.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,6 +42,50 @@ def check(result, condition, lineno):
                      condition)
 
 
+def test_constants():
+    """Test constants."""
+    check(weechat.WEECHAT_RC_OK == 0)
+    check(weechat.WEECHAT_RC_OK_EAT == 1)
+    check(weechat.WEECHAT_RC_ERROR == -1)
+    check(weechat.WEECHAT_CONFIG_READ_OK == 0)
+    check(weechat.WEECHAT_CONFIG_READ_MEMORY_ERROR == -1)
+    check(weechat.WEECHAT_CONFIG_READ_FILE_NOT_FOUND == -2)
+    check(weechat.WEECHAT_CONFIG_WRITE_OK == 0)
+    check(weechat.WEECHAT_CONFIG_WRITE_ERROR == -1)
+    check(weechat.WEECHAT_CONFIG_WRITE_MEMORY_ERROR == -2)
+    check(weechat.WEECHAT_CONFIG_OPTION_SET_OK_CHANGED == 2)
+    check(weechat.WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE == 1)
+    check(weechat.WEECHAT_CONFIG_OPTION_SET_ERROR == 0)
+    check(weechat.WEECHAT_CONFIG_OPTION_SET_OPTION_NOT_FOUND == -1)
+    check(weechat.WEECHAT_CONFIG_OPTION_UNSET_OK_NO_RESET == 0)
+    check(weechat.WEECHAT_CONFIG_OPTION_UNSET_OK_RESET == 1)
+    check(weechat.WEECHAT_CONFIG_OPTION_UNSET_OK_REMOVED == 2)
+    check(weechat.WEECHAT_CONFIG_OPTION_UNSET_ERROR == -1)
+    check(weechat.WEECHAT_LIST_POS_SORT == 'sort')
+    check(weechat.WEECHAT_LIST_POS_BEGINNING == 'beginning')
+    check(weechat.WEECHAT_LIST_POS_END == 'end')
+    check(weechat.WEECHAT_HOTLIST_LOW == '0')
+    check(weechat.WEECHAT_HOTLIST_MESSAGE == '1')
+    check(weechat.WEECHAT_HOTLIST_PRIVATE == '2')
+    check(weechat.WEECHAT_HOTLIST_HIGHLIGHT == '3')
+    check(weechat.WEECHAT_HOOK_PROCESS_RUNNING == -1)
+    check(weechat.WEECHAT_HOOK_PROCESS_ERROR == -2)
+    check(weechat.WEECHAT_HOOK_CONNECT_OK == 0)
+    check(weechat.WEECHAT_HOOK_CONNECT_ADDRESS_NOT_FOUND == 1)
+    check(weechat.WEECHAT_HOOK_CONNECT_IP_ADDRESS_NOT_FOUND == 2)
+    check(weechat.WEECHAT_HOOK_CONNECT_CONNECTION_REFUSED == 3)
+    check(weechat.WEECHAT_HOOK_CONNECT_PROXY_ERROR == 4)
+    check(weechat.WEECHAT_HOOK_CONNECT_LOCAL_HOSTNAME_ERROR == 5)
+    check(weechat.WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR == 6)
+    check(weechat.WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR == 7)
+    check(weechat.WEECHAT_HOOK_CONNECT_MEMORY_ERROR == 8)
+    check(weechat.WEECHAT_HOOK_CONNECT_TIMEOUT == 9)
+    check(weechat.WEECHAT_HOOK_CONNECT_SOCKET_ERROR == 10)
+    check(weechat.WEECHAT_HOOK_SIGNAL_STRING == 'string')
+    check(weechat.WEECHAT_HOOK_SIGNAL_INT == 'int')
+    check(weechat.WEECHAT_HOOK_SIGNAL_POINTER == 'pointer')
+
+
 def test_plugins():
     """Test plugins functions."""
     check(weechat.plugin_get_name('') == 'core')
@@ -199,10 +243,8 @@ def option_delete_cb(data, option):
 def test_config():
     """Test config functions."""
     # config
-    ptr_config = weechat.config_new(
-        'test_config_' + '{SCRIPT_LANGUAGE}',
-        'config_reload_cb', 'config_reload_data',
-    )
+    ptr_config = weechat.config_new('test_config_' + '{SCRIPT_LANGUAGE}',
+                                    'config_reload_cb', 'config_reload_data')
     check(ptr_config != '')
     # set version
     weechat.config_set_version(ptr_config, 2,
@@ -237,6 +279,18 @@ def test_config():
     check(weechat.config_option_reset(ptr_opt_bool, 1) == 2)  # SET_OK_CHANGED
     check(weechat.config_option_reset(ptr_opt_bool, 1) == 1)  # SET_OK_SAME_VALUE
     check(weechat.config_boolean(ptr_opt_bool) == 1)
+    # boolean option with parent option
+    ptr_opt_bool_child = weechat.config_new_option(
+        ptr_config, ptr_section,
+        'option_bool_child << test_config_' + '{SCRIPT_LANGUAGE}' + '.section1.option_bool',
+        'boolean', 'bool option', '', 0, 0, None, None, 1,
+        'option_check_value_cb', '',
+        'option_change_cb', '',
+        'option_delete_cb', '',
+    )
+    check(ptr_opt_bool_child != '')
+    check(weechat.config_boolean(ptr_opt_bool_child) == 0)
+    check(weechat.config_boolean_inherited(ptr_opt_bool_child) == 1)
     # integer option
     ptr_opt_int = weechat.config_new_option(
         ptr_config, ptr_section, 'option_int', 'integer', 'int option',
@@ -254,6 +308,18 @@ def test_config():
     check(weechat.config_option_reset(ptr_opt_int, 1) == 2)  # SET_OK_CHANGED
     check(weechat.config_option_reset(ptr_opt_int, 1) == 1)  # SET_OK_SAME_VALUE
     check(weechat.config_integer(ptr_opt_int) == 2)
+    # integer option with parent option
+    ptr_opt_int_child = weechat.config_new_option(
+        ptr_config, ptr_section,
+        'option_int_child << test_config_' + '{SCRIPT_LANGUAGE}' + '.section1.option_int',
+        'integer', 'int option', '', 0, 256, None, None, 1,
+        'option_check_value_cb', '',
+        'option_change_cb', '',
+        'option_delete_cb', '',
+    )
+    check(ptr_opt_int_child != '')
+    check(weechat.config_integer(ptr_opt_int_child) == 0)
+    check(weechat.config_integer_inherited(ptr_opt_int_child) == 2)
     # integer option (with string values: enum with WeeChat >= 4.1.0)
     ptr_opt_int_str = weechat.config_new_option(
         ptr_config, ptr_section, 'option_int_str', 'integer', 'int option str',
@@ -275,6 +341,19 @@ def test_config():
     check(weechat.config_option_reset(ptr_opt_int_str, 1) == 1)  # SET_OK_SAME_VALUE
     check(weechat.config_integer(ptr_opt_int_str) == 1)
     check(weechat.config_string(ptr_opt_int_str) == 'val2')
+    # integer option with parent option (with string values: enum with WeeChat >= 4.1.0)
+    ptr_opt_int_str_child = weechat.config_new_option(
+        ptr_config, ptr_section,
+        'option_int_str_child << test_config_' + '{SCRIPT_LANGUAGE}' + '.section1.option_int_str',
+        'integer', 'int option str',
+        'val1|val2|val3', 0, 0, None, None, 1,
+        'option_check_value_cb', '',
+        'option_change_cb', '',
+        'option_delete_cb', '',
+    )
+    check(ptr_opt_int_str_child != '')
+    check(weechat.config_integer(ptr_opt_int_str_child) == 0)
+    check(weechat.config_integer_inherited(ptr_opt_int_str_child) == 1)
     # string option
     ptr_opt_str = weechat.config_new_option(
         ptr_config, ptr_section, 'option_str', 'string', 'str option',
@@ -301,6 +380,18 @@ def test_config():
     check(weechat.config_option_unset(ptr_opt_str) == 0)  # UNSET_OK_NO_RESET
     check(weechat.config_string(ptr_opt_str) == 'value')
     check(weechat.config_option_default_is_null(ptr_opt_str) == 0)
+    # string option with parent option
+    ptr_opt_str_child = weechat.config_new_option(
+        ptr_config, ptr_section,
+        'option_str_child << test_config_' + '{SCRIPT_LANGUAGE}' + '.section1.option_str',
+        'string', 'str option', '', 0, 0, None, None, 1,
+        'option_check_value_cb', '',
+        'option_change_cb', '',
+        'option_delete_cb', '',
+    )
+    check(ptr_opt_str_child != '')
+    check(weechat.config_string(ptr_opt_str_child) == '')
+    check(weechat.config_string_inherited(ptr_opt_str_child) == 'value')
     # color option
     ptr_opt_col = weechat.config_new_option(
         ptr_config, ptr_section, 'option_col', 'color', 'col option',
@@ -318,6 +409,18 @@ def test_config():
     check(weechat.config_option_reset(ptr_opt_col, 1) == 2)  # SET_OK_CHANGED
     check(weechat.config_option_reset(ptr_opt_col, 1) == 1)  # SET_OK_SAME_VALUE
     check(weechat.config_color(ptr_opt_col) == 'lightgreen')
+    # color option with parent option
+    ptr_opt_col_child = weechat.config_new_option(
+        ptr_config, ptr_section,
+        'option_col_child << test_config_' + '{SCRIPT_LANGUAGE}' + '.section1.option_col',
+        'color', 'col option', '', 0, 0, None, None, 1,
+        'option_check_value_cb', '',
+        'option_change_cb', '',
+        'option_delete_cb', '',
+    )
+    check(ptr_opt_col_child != '')
+    check(weechat.config_color(ptr_opt_col_child) == '')
+    check(weechat.config_color_inherited(ptr_opt_col_child) == 'lightgreen')
     # enum option
     ptr_opt_enum = weechat.config_new_option(
         ptr_config, ptr_section, 'option_enum', 'enum', 'enum option',
@@ -343,6 +446,18 @@ def test_config():
     check(weechat.config_enum(ptr_opt_enum) == 1)
     check(weechat.config_integer(ptr_opt_enum) == 1)
     check(weechat.config_string(ptr_opt_enum) == 'val2')
+    # enum option with parent option
+    ptr_opt_enum_child = weechat.config_new_option(
+        ptr_config, ptr_section,
+        'option_enum_child << test_config_' + '{SCRIPT_LANGUAGE}' + '.section1.option_enum',
+        'enum', 'enum option', 'val1|val2|val3', 0, 0, None, None, 1,
+        'option_check_value_cb', '',
+        'option_change_cb', '',
+        'option_delete_cb', '',
+    )
+    check(ptr_opt_enum_child != '')
+    check(weechat.config_enum(ptr_opt_enum_child) == 0)
+    check(weechat.config_enum_inherited(ptr_opt_enum_child) == 1)
     # search option
     ptr_opt_bool2 = weechat.config_search_option(ptr_config, ptr_section,
                                                  'option_bool')
@@ -355,6 +470,12 @@ def test_config():
     check(weechat.config_string_to_boolean('1') == 1)
     # rename option
     weechat.config_option_rename(ptr_opt_bool, 'option_bool_renamed')
+    # get string property of option
+    check(weechat.config_option_get_string(ptr_opt_bool, 'type') == 'boolean')
+    check(weechat.config_option_get_string(ptr_opt_bool, 'name') == 'option_bool_renamed')
+    # get pointer property of option
+    check(weechat.config_option_get_pointer(ptr_opt_bool, 'config_file') == ptr_config)
+    check(weechat.config_option_get_pointer(ptr_opt_bool, 'section') == ptr_section)
     # read config (create it because it does not exist yet)
     check(weechat.config_read(ptr_config) == 0)  # CONFIG_READ_OK
     # write config
@@ -422,8 +543,8 @@ def test_display():
     weechat.prnt('', '## test print core buffer')
     weechat.prnt_date_tags('', 946681200, 'tag1,tag2',
                            '## test print_date_tags core buffer')
-    weechat.prnt_date_tags('', 2146383600, 'tag1,tag2',
-                           '## test print_date_tags core buffer, January, 6th 2038')
+    weechat.prnt_datetime_tags('', 2146383600, 123456, 'tag1,tag2',
+                               '## test print_date_tags core buffer, January, 6th 2038')
     hdata_buffer = weechat.hdata_get('buffer')
     hdata_lines = weechat.hdata_get('lines')
     hdata_line = weechat.hdata_get('line')
@@ -433,6 +554,7 @@ def test_display():
     line = weechat.hdata_pointer(hdata_lines, own_lines, 'last_line')
     data = weechat.hdata_pointer(hdata_line, line, 'data')
     check(weechat.hdata_time(hdata_line_data, data, 'date') == 2146383600)
+    check(weechat.hdata_integer(hdata_line_data, data, 'date_usec') == 123456)
     buffer = weechat.buffer_new('test_formatted',
                                 'buffer_input_cb', '', 'buffer_close_cb', '')
     check(buffer != '')
@@ -448,8 +570,8 @@ def test_display():
     weechat.prnt_y(buffer, 0, '## test print_y free buffer')
     weechat.prnt_y_date_tags(buffer, 0, 946681200, 'tag1,tag2',
                              '## test print_y_date_tags free buffer')
-    weechat.prnt_y_date_tags(buffer, 1, 2146383600, 'tag1,tag2',
-                             '## test print_y_date_tags free buffer, January, 6th 2038')
+    weechat.prnt_y_datetime_tags(buffer, 1, 2146383600, 123456, 'tag1,tag2',
+                                 '## test print_y_date_tags free buffer, January, 6th 2038')
     weechat.buffer_close(buffer)
 
 
@@ -583,8 +705,8 @@ def test_hdata():
     line2 = weechat.hdata_pointer(hdata_line, line1, 'next_line')
     line3 = weechat.hdata_pointer(hdata_line, line2, 'next_line')
     # hdata_get_var_offset
-    check(weechat.hdata_get_var_offset(hdata_buffer, 'plugin') == 0)
-    check(weechat.hdata_get_var_offset(hdata_buffer, 'number') > 0)
+    check(weechat.hdata_get_var_offset(hdata_buffer, 'id') == 0)
+    check(weechat.hdata_get_var_offset(hdata_buffer, 'plugin') > 0)
     # hdata_get_var_type_string
     check(weechat.hdata_get_var_type_string(hdata_buffer, 'plugin') == 'pointer')
     check(weechat.hdata_get_var_type_string(hdata_buffer, 'number') == 'integer')
@@ -593,7 +715,8 @@ def test_hdata():
     check(weechat.hdata_get_var_type_string(hdata_line_data, 'displayed') == 'char')
     check(weechat.hdata_get_var_type_string(hdata_line_data, 'prefix') == 'shared_string')
     check(weechat.hdata_get_var_type_string(hdata_line_data, 'date') == 'time')
-    check(weechat.hdata_get_var_type_string(hdata_hotlist, 'creation_time.tv_usec') == 'long')
+    check(weechat.hdata_get_var_type_string(hdata_hotlist, 'time') == 'time')
+    check(weechat.hdata_get_var_type_string(hdata_hotlist, 'time_usec') == 'long')
     check(weechat.hdata_get_var_type_string(hdata_irc_server, 'sasl_scram_salted_pwd') == 'other')
     # hdata_get_var_array_size
     check(weechat.hdata_get_var_array_size(hdata_buffer, buffer2, 'name') == -1)
@@ -629,6 +752,8 @@ def test_hdata():
     weechat.buffer_set(buffer, 'hotlist', weechat.WEECHAT_HOTLIST_MESSAGE)
     gui_hotlist = weechat.hdata_get_list(hdata_hotlist, 'gui_hotlist')
     check(weechat.hdata_long(hdata_hotlist, gui_hotlist, 'creation_time.tv_usec') >= 0)
+    # hdata_longlong
+    check(weechat.hdata_longlong(hdata_buffer, buffer2, 'id') > 1708874542000000)
     # hdata_string
     check(weechat.hdata_string(hdata_buffer, buffer2, 'name') == 'test')
     # hdata_pointer
@@ -669,6 +794,7 @@ def cmd_test_cb(data, buf, args):
     weechat.prnt('', '>>> ------------------------------')
     weechat.prnt('', '>>> Testing ' + '{SCRIPT_LANGUAGE}' + ' API')
     weechat.prnt('', '  > TESTS: ' + '{SCRIPT_TESTS}')
+    test_constants()
     test_plugins()
     test_strings()
     test_lists()
