@@ -1,7 +1,7 @@
 /*
  * test-irc-tag.cpp - test IRC message tags functions
  *
- * Copyright (C) 2021-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2021-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -26,8 +26,8 @@
 extern "C"
 {
 #include <stdio.h>
-#include "src/core/wee-hashtable.h"
-#include "src/core/wee-hook.h"
+#include "src/core/core-hashtable.h"
+#include "src/core/core-hook.h"
 #include "src/plugins/irc/irc-tag.h"
 #include "src/plugins/plugin.h"
 
